/****************************************************************************
*  This file is part of the blinky program for the TNKernel RTOS
*
*  Copyright (c) 2009 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
****************************************************************************
*  History:
*
*  19.09.2009  mifi  First Version
****************************************************************************/
#if !defined(__ARCH_H__)
#define __ARCH_H__

/*
 * This is the architecture depending include file for the different cpu 
 * families. Dependent from the TN_ARCH_XXX and TN_PORT_XXX the correct
 * header file will be included.
 */

/*
 * Check for CrossWorks for ARM
 */
#if defined(__CROSSWORKS_ARM)

#if defined(TN_ARCH_ARM)         /* Check if the ARM architecture will be used */

#if defined(TN_PORT_SAM7)        /* SAM7 */
#include "targets/AT91SAM7.h"
#endif

#if defined(TN_PORT_LPC2000)     /* LPC2000 */
#include "targets/lpc2000.h"
#endif

#if defined(TN_PORT_STR71X)      /* STR71X */
#include "targets/str71x.h"
#endif

#endif /* TN_ARCH_ARM */         /* End of ARM architecture */


#if defined(TN_ARCH_CM3)         /* Check if the Cortex-M3 architecture will be used */

#if defined(TN_PORT_SAM3)        /* SAM3 */
#include "sam3.h"
#endif

#if defined(TN_PORT_LPC1000)     /* LPC1000 */
#include "lpc1000.h"
#endif

#if defined(TN_PORT_STM32)       /* STM32 */
#include "targets/stm32f10x.h"
#endif

#endif /* TN_ARCH_CM3 */         /* End of the Cortex-M3 architecture */

#endif /* __CROSSWORKS_ARM */

#endif /* !__ARCH_H__ */
/*** EOF ***/
