/*****************************************************************************
 * Copyright (c) 2009 Rowley Associates Limited.                             *
 *                                                                           *
 * This file may be distributed under the terms of the License Agreement     *
 * provided with this software.                                              *
 *                                                                           *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE   *
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. *
 *****************************************************************************/

/*****************************************************************************
 *                           Preprocessor Definitions
 *                           ------------------------
 *
 * NO_WATCHDOG_DISABLE
 *
 *   If defined, the watchdog will not be disabled.
 *
 * NO_USER_RESET
 *
 *   If defined, user reset will not be enabled.
 *
 * EEFC_FMR_FWS_VAL
 *
 *   Override the default Flash Mode Register FWS value by defining
 *   EEFC_FMR_FWS_VAL.
 *
 * CKGR_MOR_VAL
 *
 *   Override the default Main Oscillator Register value by defining
 *   CKGR_MOR_VAL.
 *
 * CKGR_PLLAR_VAL
 *
 *   Override the default PLLA configuration value by defining CKGR_PLLAR_VAL.
 *
 * CKGR_UCKR_VAL
 *
 *   Override the default UTMI clock configuration value by defining 
 *   CKGR_UCKR_VAL.
 *
 *****************************************************************************/

#include <SAM3.h>

#ifndef EEFC_FMR_FWS_VAL
#define EEFC_FMR_FWS_VAL 2
#endif

#ifndef CKGR_MOR_VAL
#define CKGR_MOR_VAL (0x37 << CKGR_MOR_KEY_BIT) | (0x3F << CKGR_MOR_MOSCXTST_BIT) | CKGR_MOR_MOSCRCEN | CKGR_MOR_MOSCXTEN
#endif

#ifndef CKGR_PLLAR_VAL
#if OSCILLATOR_CLOCK_FREQUENCY==12000000
/* PLLACK = 12MHz * 8 / 1 = 96MHz */
#define CKGR_PLLAR_VAL (1 << 29) | (7 << CKGR_PLLAR_MULA_BIT) | (0 << CKGR_PLLAR_STMODE_BIT) | (63 << CKGR_PLLAR_PLLACOUNT_BIT) | (1 << CKGR_PLLAR_DIVA_BIT)
#define PMC_MCKR_PRES_VAL (0 << PMC_MCKR_PRES_BIT)
#else
#error OSCILLATOR_CLOCK_FREQUENCY not supported
#endif
#endif

#ifndef CKGR_UCKR_VAL
#define CKGR_UCKR_VAL (3 << CKGR_UCKR_UPLLCOUNT_BIT) | CKGR_UCKR_UPLLEN
#endif

  .syntax unified
  .section .init, "ax"
  .code 16
  .align 0
  .global _configure

  .thumb_func

_configure:
#ifndef NO_WATCHDOG_DISABLE
  /* Disable Watchdog */
  ldr r1, =WDT_BASE_ADDRESS
  ldr r0, =WDT_MR_WDDIS
  str r0, [r1, #WDT_MR_BASE_OFFSET]
#endif

#ifndef NO_USER_RESET
  /* Enable user reset */
  ldr r1, =RSTC_BASE_ADDRESS
  ldr r0, =0xA5000001
  str r0, [r1, #RSTC_MR_BASE_OFFSET]
#endif

  /* Set up FLASH wait state */
  ldr r0, =(EEFC_FMR_FWS_VAL << EEFC0_FMR_FWS_BIT)
  ldr r1, =EEFC0_BASE_ADDRESS
  str r0, [r1, #EEFC0_FMR_BASE_OFFSET]
#ifdef EEFC1_BASE_ADDRESS
  ldr r1, =EEFC1_BASE_ADDRESS
  str r0, [r1, #EEFC1_FMR_BASE_OFFSET]
#endif

  /* Select external slow clock */
  ldr r1, =SUPC_BASE_ADDRESS
  ldr r0, [r1, #SUPC_SR_BASE_OFFSET]
  tst r0, #SUPC_SR_OSCSEL
  bne 2f
  ldr r0, =(0xA5000000 | SUPC_CR_XTALSEL)
  str r0, [r1, #SUPC_CR_BASE_OFFSET]
1:
  ldr r0, [r1, #SUPC_SR_BASE_OFFSET]
  tst r0, #SUPC_SR_OSCSEL
  beq 1b
2:

  /* Enable the main oscillator */
  ldr r1, =PMC_BASE_ADDRESS
  ldr r0, =CKGR_MOR_VAL
  str r0, [r1, #CKGR_MOR_BASE_OFFSET]
1:
  ldr r0, [r1, #PMC_SR_BASE_OFFSET]
  tst r0, #PMC_SR_MOSCXTS
  beq 1b

  /* Select main oscillator */
  ldr r0, =(CKGR_MOR_VAL | CKGR_MOR_MOSCSEL)
  str r0, [r1, #CKGR_MOR_BASE_OFFSET]
1:
  ldr r0, [r1, #PMC_SR_BASE_OFFSET]
  tst r0, #PMC_SR_MOSCSELS
  beq 1b
  ldr r0, [r1, #PMC_MCKR_BASE_OFFSET]
  bic r0, r0, #PMC_MCKR_CSS_MASK
  orr r0, r0, #(1 << PMC_MCKR_CSS_BIT)
  str r0, [r1, #PMC_MCKR_BASE_OFFSET]
1:
  ldr r0, [r1, #PMC_SR_BASE_OFFSET]
  tst r0, #PMC_SR_MCKRDY
  beq 1b

  /* Set up the PLL */
  ldr r0, =CKGR_PLLAR_VAL
  str r0, [r1, #CKGR_PLLAR_BASE_OFFSET]
1:/* Wait for PLL to lock */
  ldr r0, [r1, #PMC_SR_BASE_OFFSET]
  tst r0, #PMC_SR_LOCKA
  beq 1b

  /* Set up the USB PLL */
  ldr r0, =CKGR_UCKR_VAL
  str r0, [r1, #CKGR_UCKR_BASE_OFFSET]
1:/* Wait for PLL to lock */
  ldr r0, [r1, #PMC_SR_BASE_OFFSET]
  tst r0, #PMC_SR_LOCKU
  beq 1b

  /* Select PLL as clock source */
  ldr r0, =(PMC_MCKR_PRES_VAL | 1 << PMC_MCKR_CSS_BIT)
  str r0, [r1, #PMC_MCKR_BASE_OFFSET]
1:/* Wait for MCKRDY */
  ldr r0, [r1, #PMC_SR_BASE_OFFSET]
  tst r0, #PMC_SR_MCKRDY
  beq 1b
  
  ldr r0, =(PMC_MCKR_PRES_VAL | 2 << PMC_MCKR_CSS_BIT)
  str r0, [r1, #PMC_MCKR_BASE_OFFSET]
1:/* Wait for MCKRDY */
  ldr r0, [r1, #PMC_SR_BASE_OFFSET]
  tst r0, #PMC_SR_MCKRDY
  beq 1b

  ldr r1, =MATRIX_BASE_ADDRESS
  /* Set Cortex-M3 System master to be Internal SRAM's default master */
  ldr r0, =(2 << MATRIX_SCFG0_DEFMSTR_TYPE_BIT) | (1 << MATRIX_SCFG0_FIXED_DEFMSTR_BIT)
  str r0, [r1, #MATRIX_SCFG0_BASE_OFFSET]
  str r0, [r1, #MATRIX_SCFG1_BASE_OFFSET]
  /* Set Cortex-M3 Instruction/Data master to be Internal FLASH's default master */
  ldr r0, =(2 << MATRIX_SCFG0_DEFMSTR_TYPE_BIT) | (0 << MATRIX_SCFG0_FIXED_DEFMSTR_BIT)
  str r0, [r1, #MATRIX_SCFG3_BASE_OFFSET]
  str r0, [r1, #MATRIX_SCFG4_BASE_OFFSET]

  bx lr

