/*****************************************************************************
 * Copyright (c) 2009 Rowley Associates Limited.                             *
 *                                                                           *
 * This file may be distributed under the terms of the License Agreement     *
 * provided with this software.                                              *
 *                                                                           *
 * THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE   *
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. *
 *****************************************************************************/

/*****************************************************************************
 *                           Preprocessor Definitions
 *                           ------------------------
 *
 * STARTUP_FROM_RESET
 *
 *   If defined, the program will startup from power-on/reset. If not defined
 *   the program will just loop endlessly from power-on/reset.
 *
 *   This definition is not defined by default on this target because the
 *   debugger is unable to reset this target and maintain control of it over the
 *   JTAG interface. The advantage of doing this is that it allows the debugger
 *   to reset the CPU and run programs from a known reset CPU state on each run.
 *   It also acts as a safety net if you accidently download a program in FLASH
 *   that crashes and prevents the debugger from taking control over JTAG
 *   rendering the target unusable over JTAG. The obvious disadvantage of doing
 *   this is that your application will not startup without the debugger.
 *
 *   We advise that on this target you keep STARTUP_FROM_RESET undefined whilst
 *   you are developing and only define STARTUP_FROM_RESET when development is
 *   complete.
 *
 *****************************************************************************/

  .global reset_handler

  .syntax unified

  .section .vectors, "ax"
  .code 16
  .align 0
  .global _vectors

.macro DEFAULT_ISR_HANDLER name=
  .thumb_func
  .weak \name
\name:
1: b 1b /* endless loop */
.endm

_vectors:
  .word __stack_end__
#ifdef STARTUP_FROM_RESET
  .word reset_handler
#else
  .word reset_wait
#endif /* STARTUP_FROM_RESET */
  .word NMI_Handler
  .word HardFault_Handler
  .word MemManage_Handler
  .word BusFault_Handler
  .word UsageFault_Handler
  .word 0 // Reserved
  .word 0 // Reserved
  .word 0 // Reserved
  .word 0 // Reserved
  .word SVC_Handler
  .word DebugMon_Handler
  .word 0 // Reserved
  .word PendSV_Handler
  .word SysTick_Handler 
  .word SUPC_IrqHandler
  .word RSTC_IrqHandler
  .word RTC_IrqHandler
  .word RTT_IrqHandler
  .word WDT_IrqHandler
  .word PMC_IrqHandler
  .word EFC0_IrqHandler
  .word EFC1_IrqHandler
  .word DBGU_IrqHandler
  .word HSMC4_IrqHandler
  .word PIOA_IrqHandler
  .word PIOB_IrqHandler
  .word PIOC_IrqHandler
  .word USART0_IrqHandler
  .word USART1_IrqHandler
  .word USART2_IrqHandler
  .word USART3_IrqHandler
  .word MCI0_IrqHandler
  .word TWI0_IrqHandler
  .word TWI1_IrqHandler
  .word SPI0_IrqHandler
  .word SSC0_IrqHandler
  .word TC0_IrqHandler
  .word TC1_IrqHandler
  .word TC2_IrqHandler
  .word PWM_IrqHandler
  .word ADCC0_IrqHandler
  .word ADCC1_IrqHandler
  .word HDMA_IrqHandler
  .word UDPD_IrqHandler

  .section .init, "ax"
  .thumb_func

  reset_handler:
#ifndef __FLASH_BUILD
  /* If this is a RAM build, configure vector table offset register to point 
     to the RAM vector table. */
  ldr r0, =0xE000ED08
  ldr r1, =_vectors
  str r1, [r0]
#endif

  bl _configure

  b _start

DEFAULT_ISR_HANDLER NMI_Handler
DEFAULT_ISR_HANDLER HardFault_Handler
DEFAULT_ISR_HANDLER MemManage_Handler
DEFAULT_ISR_HANDLER BusFault_Handler
DEFAULT_ISR_HANDLER UsageFault_Handler
DEFAULT_ISR_HANDLER SVC_Handler 
DEFAULT_ISR_HANDLER DebugMon_Handler
DEFAULT_ISR_HANDLER PendSV_Handler
DEFAULT_ISR_HANDLER SysTick_Handler 
DEFAULT_ISR_HANDLER SUPC_IrqHandler
DEFAULT_ISR_HANDLER RSTC_IrqHandler
DEFAULT_ISR_HANDLER RTC_IrqHandler
DEFAULT_ISR_HANDLER RTT_IrqHandler
DEFAULT_ISR_HANDLER WDT_IrqHandler
DEFAULT_ISR_HANDLER PMC_IrqHandler
DEFAULT_ISR_HANDLER EFC0_IrqHandler
DEFAULT_ISR_HANDLER EFC1_IrqHandler
DEFAULT_ISR_HANDLER DBGU_IrqHandler
DEFAULT_ISR_HANDLER HSMC4_IrqHandler
DEFAULT_ISR_HANDLER PIOA_IrqHandler
DEFAULT_ISR_HANDLER PIOB_IrqHandler
DEFAULT_ISR_HANDLER PIOC_IrqHandler
DEFAULT_ISR_HANDLER USART0_IrqHandler
DEFAULT_ISR_HANDLER USART1_IrqHandler
DEFAULT_ISR_HANDLER USART2_IrqHandler
DEFAULT_ISR_HANDLER USART3_IrqHandler
DEFAULT_ISR_HANDLER MCI0_IrqHandler
DEFAULT_ISR_HANDLER TWI0_IrqHandler
DEFAULT_ISR_HANDLER TWI1_IrqHandler
DEFAULT_ISR_HANDLER SPI0_IrqHandler
DEFAULT_ISR_HANDLER SSC0_IrqHandler
DEFAULT_ISR_HANDLER TC0_IrqHandler
DEFAULT_ISR_HANDLER TC1_IrqHandler
DEFAULT_ISR_HANDLER TC2_IrqHandler
DEFAULT_ISR_HANDLER PWM_IrqHandler
DEFAULT_ISR_HANDLER ADCC0_IrqHandler
DEFAULT_ISR_HANDLER ADCC1_IrqHandler
DEFAULT_ISR_HANDLER HDMA_IrqHandler
DEFAULT_ISR_HANDLER UDPD_IrqHandler

#ifndef STARTUP_FROM_RESET
DEFAULT_ISR_HANDLER reset_wait
#endif /* STARTUP_FROM_RESET */

